--[[
The ShellCommand implements 'Turbine.ShellCommand'.
The ShellCommand use a ShellCommandHandler for the work.
]]

ShellCommand = class(Turbine.ShellCommand);

-- Initializes a new ShellCommand instance
function ShellCommand:Constructor(commandHandler)
	Turbine.ShellCommand.Constructor(self);
	
	self.Handler = commandHandler;
	self.RegisteredCommandName = nil;
end

-- Override 'Turbine.ShellCommand'
-- Executes the registered shell command (for example a 'spam' command). 
-- The really commands used by the ShellCommandHandler are the arguments.
function ShellCommand:Execute(command, arguments)
	local cmdTable = {};
	local cmdLine  = "";

	for word in string.gfind(arguments, "(%w+)") do
		-- build the command line
		if cmdLine == "" then
			cmdLine = word;
		else
			cmdLine = cmdLine .. " " .. word;
		end

		-- validate the command line by the handler
		if self.Handler:ValidateCommand(cmdLine) == true then
			table.insert(cmdTable, cmdLine);
			cmdLine = "";
		end
	end

	-- should be empty, except command line is invalid
	if cmdLine ~= "" then
		self:DisplayInvalidCommand(command, arguments);
		return;
	end

	-- execute all valid commands
	for _, cmd in pairs(cmdTable) do
		if self.Handler:ExecuteCommand(cmd) ~= true then
			self:DisplayInvalidCommand(command, arguments);
			return;
		end
	end
end

function ShellCommand:TextOut(text)
	Turbine.Shell.WriteLine("Command:> " .. text);
end

function ShellCommand:DisplayInvalidCommand(command, arguments)
	self:TextOut("invalid command: /" .. command .. " " .. arguments);
	self:TextOut("see help: " .. self:GetShortHelp());
end

-- Override 'Turbine.ShellCommand'
function ShellCommand:GetHelp()
	return self:GetShortHelp();
end

-- Override 'Turbine.ShellCommand'
function ShellCommand:GetShortHelp(argumentText)
	return "/" .. tostring(self.RegisteredCommandName) .. " " .. self.Handler:GetShortHelp();
end

-- Register this ShellCommand instance for the alias
function ShellCommand:RegisterCommandName(name)
	if self.RegisteredCommandName ~= nil then
		self:UnregisterCommandName();
	end
	
	if Turbine.Shell.IsCommand(name) == false then
		Turbine.Shell.AddCommand(name, self);
		self.RegisteredCommandName = name;

		Turbine.Shell.WriteLine("The shell command <rgb=#00b9b7>/" .. self.RegisteredCommandName .."</rgb> was added (see <rgb=#00b9b7>/" .. Strings.HelpCommandName .. " " .. self.RegisteredCommandName .."</rgb>)");
		return true;
	end

	Turbine.Shell.WriteLine("Unable to register the command name <rgb=#00b9b7>/" .. name .."</rgb>");
	return false;
end

-- Unregister this ShellCommand instance
function ShellCommand:UnregisterCommandName()
	if self.RegisteredCommandName ~= nil then
		Turbine.Shell.RemoveCommand(self);
		Turbine.Shell.WriteLine("The shell command <rgb=#00b9b7>/" .. self.RegisteredCommandName .."</rgb> was removed");
		self.RegisteredCommandName = nil;
	end
end
